package nc.vo.util;

package ..deTemp.1736863183571;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import nc.bs.framework.common.NCLocator;
import nc.bs.logging.Logger;
import nc.bs.mw.sqltrans.TempTable;
import nc.bs.uif2.VersionConflictException;
import nc.itf.uap.IUAPQueryBS;
import nc.jdbc.framework.JdbcSession;
import nc.jdbc.framework.PersistenceManager;
import nc.jdbc.framework.SQLParameter;
import nc.jdbc.framework.exception.DbException;
import nc.jdbc.framework.processor.ArrayListProcessor;
import nc.jdbc.framework.processor.ColumnProcessor;
import nc.jdbc.framework.processor.ResultSetProcessor;
import nc.md.MDBaseQueryFacade;
import nc.md.data.access.NCObject;
import nc.md.model.IBean;
import nc.vo.jcom.lang.StringUtil;
import nc.vo.ml.NCLangRes4VoTransl;
import nc.vo.pub.AggregatedValueObject;
import nc.vo.pub.BusinessException;
import nc.vo.pub.CircularlyAccessibleValueObject;
import nc.vo.pub.SuperVO;
import nc.vo.trade.sqlutil.IInSqlBatchCallBack;
import nc.vo.trade.sqlutil.InSqlBatchCaller;
import nccloud.commons.lang.StringUtils;

public class BDVersionValidationUtil {
  public static void validateSuperVO(SuperVO... superVOs) throws BusinessException {
    if (superVOs == null)
      throw new BusinessException(NCLangRes4VoTransl.getNCLangRes()
          .getStrByID("ueaa", "BDVersionValidationUtil-000000")); 
    Map<String, List<String[]>> pktsByTable = new HashMap<>();
    for (SuperVO superVO : superVOs) {
      String tableAndPKField = superVO.getTableName() + ":" + superVO.getPKFieldName();
      String pkValue = superVO.getPrimaryKey();
      if (pkValue != null && !StringUtils.isEmpty(pkValue)) {
        List<String[]> pks = pktsByTable.get(tableAndPKField);
        if (pks == null) {
          pks = (List)new ArrayList<>();
          pktsByTable.put(tableAndPKField, pks);
        } 
        pks.add(new String[] { pkValue, 
              
              (superVO.getAttributeValue("ts") == null) ? null : superVO
              .getAttributeValue("ts").toString() });
      } 
    } 
    String columnNms = "pk varchar(20), ts char(19)";
    String tempTableNm = null;
    PersistenceManager manager = null;
    try {
      manager = PersistenceManager.getInstance();
      JdbcSession session = manager.getJdbcSession();
      tempTableNm = createTempTable("temppkts", columnNms, "ts", session);
      Iterator<String> iterator = pktsByTable.keySet().iterator();
      while (iterator.hasNext()) {
        String tableAndPkField = iterator.next();
        String[] temp = StringUtils.split(tableAndPkField, ":");
        validateSuperVOBatch2(temp[0], temp[1], pktsByTable
            .get(tableAndPkField), tempTableNm, session);
      } 
    } catch (DbException e) {
      Logger.error(e.getMessage(), (Throwable)e);
      throw new BusinessException(e.getMessage());
    } finally {
      if (manager != null)
        manager.release(); 
    } 
  }
  
  private static void validateSuperVOBatch2(String table, String pkField, List<String[]> pktsList, String tempTable, JdbcSession session) throws BusinessException {
    Map<String, String> pkTsFromDB;
    String deleteSql = "delete from " + tempTable;
    try {
      session.executeUpdate(deleteSql);
    } catch (DbException e) {
      Logger.error(e.getMessage(), (Throwable)e);
      throw new BusinessException(e.getMessage());
    } 
    String insertSql = "insert into " + tempTable + "(pk, ts) values(?, ?)";
    List<SQLParameter> params = new ArrayList<>();
    for (String[] pkts : pktsList) {
      SQLParameter param = new SQLParameter();
      param.addParam(pkts[0]);
      param.addParam(pkts[1]);
      params.add(param);
    } 
    try {
      session.addBatch(insertSql, params.<SQLParameter>toArray(new SQLParameter[0]));
      session.executeBatch();
    } catch (DbException e) {
      Logger.error(e.getMessage(), (Throwable)e);
      throw new BusinessException(e.getMessage());
    } 
    String selectSql = "select t3.pk,t3.ts from (select t2.*, t1.ts as t1ts from " + tempTable + " t2 left join " + table + " t1 on (t1." + pkField + "=t2.pk and (t1.ts=t2.ts or (t1.ts is not null and t2.ts is null)))) t3 where t3.t1ts is null;";
    try {
      pkTsFromDB = (Map<String, String>)session.executeQuery(selectSql, (ResultSetProcessor)new Object());
    } catch (DbException e) {
      Logger.error(e.getMessage(), (Throwable)e);
      throw new BusinessException(e.getMessage());
    } 
    if (pkTsFromDB.size() > 0)
      throw new VersionConflictException(getUpdateInfo()); 
  }
  
  private static void validateSuperVOBatch(String table, String pkField, List<String[]> pktsList) throws BusinessException {
    int pageCount = pktsList.size() / 500 + 1;
    for (int page = 0; page < pageCount; page++) {
      StringBuilder sqlIn = new StringBuilder();
      for (int row = page * 500; row < (page + 1) * 500 && row < pktsList
        .size(); row++) {
        String pkValue = ((String[])pktsList.get(row))[0];
        sqlIn.append("'");
        sqlIn.append(pkValue);
        sqlIn.append("'");
        sqlIn.append(",");
      } 
      if (sqlIn.length() >= 1) {
        sqlIn.deleteCharAt(sqlIn.length() - 1);
        StringBuilder sql = new StringBuilder();
        sql.append("select ");
        sql.append(pkField);
        sql.append(" as pk, ts from ");
        sql.append(table);
        sql.append(" where ");
        sql.append(pkField);
        sql.append(" in(");
        sql.append(sqlIn.toString());
        sql.append(");");
        Map<String, String> pkTsFromDB = null;
        try {
          pkTsFromDB = (Map<String, String>)getQuerySer().executeQuery(sql
              .toString(), (ResultSetProcessor)new Object());
        } catch (BusinessException e) {
          Logger.error(e.getMessage(), (Throwable)e);
          throw new VersionConflictException(e.getMessage());
        } 
        if (pkTsFromDB == null)
          throw new VersionConflictException(getDelInfo()); 
        for (int i = page * 500; i < (page + 1) * 500 && i < pktsList
          .size(); i++) {
          String pkValue = ((String[])pktsList.get(i))[0];
          String ts = ((String[])pktsList.get(i))[1];
          String oldTs = pkTsFromDB.get(pkValue);
          if (oldTs == null)
            throw new VersionConflictException(getDelInfo()); 
          if (!StringUtils.isEmpty(ts))
            if (!StringUtils.equalsIgnoreCase(oldTs, ts))
              throw new VersionConflictException(getUpdateInfo());  
        } 
      } 
    } 
  }
  
  public static void validateObject(Object... objects) throws BusinessException {
    if (objects == null)
      throw new VersionConflictException(
          NCLangRes4VoTransl.getNCLangRes().getStrByID("ueaa", "BDVersionValidationUtil-000000")); 
    for (Object object : objects) {
      IBean bean = MDBaseQueryFacade.getInstance().getBeanByFullClassName(object
          .getClass().getName());
      if (bean == null)
        throw new VersionConflictException(object.getClass().getName() + 
            NCLangRes4VoTransl.getNCLangRes().getStrByID("ueaa", "BDVersionValidationUtil-000001")); 
      validateObject(bean, new Object[] { object });
    } 
  }
  
  public static void validateObject(IBean bean, Object... objects) throws BusinessException {
    if (objects == null)
      throw new VersionConflictException(
          NCLangRes4VoTransl.getNCLangRes().getStrByID("ueaa", "BDVersionValidationUtil-000000")); 
    if (bean == null)
      throw new VersionConflictException(objects[0].getClass().getName() + 
          NCLangRes4VoTransl.getNCLangRes().getStrByID("ueaa", "BDVersionValidationUtil-000001")); 
    for (Object object : objects) {
      Object oldTs = null;
      String tableName = bean.getTable().getName();
      String pkFieldName = bean.getTable().getPrimaryKeyName();
      NCObject ncObj = NCObject.newInstance(bean, object);
      try {
        Object pkObj = ncObj.getAttributeValue(bean
            .getAttributeByName(pkFieldName));
        if (pkObj != null) {
          String pkValue = pkObj.toString();
          String where = " select ts from " + tableName + " where " + pkFieldName + " ='" + pkValue + "'";
          oldTs = getQuerySer().executeQuery(where, (ResultSetProcessor)new ColumnProcessor());
          if (oldTs == null)
            throw new VersionConflictException(getDelInfo()); 
          String str1 = null;
          if (ncObj.getAttributeValue(bean.getAttributeByName("ts")) != null)
            str1 = ncObj.getAttributeValue(bean.getAttributeByName("ts")).toString(); 
          if (!StringUtil.isEmptyWithTrim(str1))
            if (!oldTs.toString().equals(str1))
              throw new VersionConflictException(getUpdateInfo());  
        } 
      } catch (BusinessException businessException) {
        Logger.error(businessException.getMessage(), (Throwable)businessException);
        throw new BusinessException(NCLangRes4VoTransl.getNCLangRes().getStrByID("ueaa", "BDVersionValidationUtil-000002"));
      } 
    } 
  }
  
  public static void validateVersion(Object obj) throws BusinessException {
    if (obj == null)
      throw new BusinessException(NCLangRes4VoTransl.getNCLangRes()
          .getStrByID("ueaa", "BDVersionValidationUtil-000000")); 
    Object[] objs = null;
    if (obj.getClass().isArray()) {
      objs = (Object[])obj;
    } else {
      objs = new Object[] { obj };
    } 
    for (Object object : objs) {
      CircularlyAccessibleValueObject headVO = null;
      CircularlyAccessibleValueObject[] childrenVOs = null;
      if (object instanceof CircularlyAccessibleValueObject) {
        headVO = (CircularlyAccessibleValueObject)object;
      } else if (object instanceof AggregatedValueObject) {
        headVO = ((AggregatedValueObject)object).getParentVO();
        childrenVOs = ((AggregatedValueObject)object).getChildrenVO();
      } 
      if (headVO != null) {
        validateHeadVO(headVO);
      } else {
        validateChildernVOs(childrenVOs);
      } 
    } 
  }
  
  private static void validateChildernVOs(CircularlyAccessibleValueObject[] childrenVOs) throws BusinessException {
    if (childrenVOs == null || childrenVOs.length == 0)
      return; 
    ArrayList<String> pkList = new ArrayList<>();
    HashMap<String, String> oldTsMap = new HashMap<>();
    HashMap<String, String> newTsMap = new HashMap<>();
    try {
      for (CircularlyAccessibleValueObject circularlyAccessibleValueObject : childrenVOs) {
        String oldTs = (String)circularlyAccessibleValueObject.getAttributeValue("ts");
        if (oldTs != null && 
          !StringUtil.isEmptyWithTrim(circularlyAccessibleValueObject.getPrimaryKey())) {
          pkList.add(circularlyAccessibleValueObject.getPrimaryKey());
          oldTsMap.put(circularlyAccessibleValueObject.getPrimaryKey(), oldTs);
        } 
      } 
      SuperVO childVO = (SuperVO)childrenVOs[0];
      StringBuilder strBuilder = new StringBuilder();
      strBuilder.append("select ");
      strBuilder.append(childVO.getPKFieldName());
      strBuilder.append(",ts from ");
      strBuilder.append(childVO.getTableName());
      strBuilder.append(" where ts in ");
      InSqlBatchCaller caller = new InSqlBatchCaller(pkList.<String>toArray(new String[0]));
      caller.execute((IInSqlBatchCallBack)new Object(strBuilder));
      List tsList = (List)getQuerySer().executeQuery(strBuilder.toString(), (ResultSetProcessor)new ArrayListProcessor());
      for (Iterator<Object[]> iter = tsList.iterator(); iter.hasNext(); ) {
        Object[] obj = iter.next();
        newTsMap.put(obj[0].toString(), obj[1].toString());
      } 
      Iterator<String> ite = oldTsMap.keySet().iterator();
      while (ite.hasNext()) {
        String key = ite.next();
        String oldTs = oldTsMap.get(key);
        String newTs = newTsMap.get(key);
        if (oldTs != null && newTs != null && !oldTs.equals(newTs))
          throw new BusinessException(getUpdateInfo()); 
      } 
    } catch (Exception e) {
      Logger.error(e.getMessage(), e);
      throw new BusinessException(e.getMessage());
    } 
  }
  
  private static void validateHeadVO(CircularlyAccessibleValueObject headVO) throws BusinessException {
    Object ts = headVO.getAttributeValue("ts");
    CircularlyAccessibleValueObject dbVO = (CircularlyAccessibleValueObject)getQuerySer().retrieveByPK(headVO
        .getClass(), headVO.getPrimaryKey(), new String[] { "ts" });
    if (dbVO == null)
      throw new BusinessException(getDelInfo()); 
    if (ts == null || StringUtil.isEmptyWithTrim(ts.toString()))
      return; 
    if (!ts.equals(dbVO.getAttributeValue("ts")))
      throw new BusinessException(getUpdateInfo()); 
  }
  
  private static IUAPQueryBS getQuerySer() {
    return (IUAPQueryBS)NCLocator.getInstance().lookup(IUAPQueryBS.class
        .getName());
  }
  
  public static String createTempTable(String tableName, String columns, String indexs, JdbcSession session) throws BusinessException {
    String vtn = null;
    try {
      vtn = (new TempTable()).createTempTable(session.getConnection(), tableName, columns, new String[] { indexs });
    } catch (SQLException e) {
      Logger.error(e.getMessage(), e);
      throw new BusinessException(e.getMessage());
    } 
    return vtn;
  }
  
  public static void execBatchSql(String sql, List<SQLParameter> list) throws BusinessException {
    PersistenceManager manager = null;
    try {
      manager = PersistenceManager.getInstance();
      JdbcSession session = manager.getJdbcSession();
      session.setAddTimeStamp(false);
      session.addBatch(sql, list.<SQLParameter>toArray(new SQLParameter[list.size()]));
      int i = session.executeBatch();
    } catch (DbException e) {
      Logger.error(e.getMessage(), (Throwable)e);
      throw new BusinessException(e.getMessage());
    } finally {
      if (manager != null)
        manager.release(); 
    } 
  }
  
  public static String getDelInfo() {
    return NCLangRes4VoTransl.getNCLangRes().getStrByID("ueaa", "BDVersionValidationUtil-000003");
  }
  
  public static String getUpdateInfo() {
    return NCLangRes4VoTransl.getNCLangRes().getStrByID("ueaa", "BDVersionValidationUtil-000004");
  }
}
